@@ -1,11 +1,15 @@
 import abc
 from typing import Generic, TypeVar, NamedTuple, Sequence, Union, Mapping, Iterable, Dict, \
-    Iterator, Any, Type, Optional  # pylint: disable=unused-import
+    Iterator, Any, Type, Optional
+
+import networkx
 
 from . import qops as q
-from .records import DataRecord, SnapshotRef
+from . import records
+from .records import DataRecord
+from . import operations
 
-__all__ = 'Archive', 'BaseArchive', 'ASCENDING', 'DESCENDING'
+__all__ = 'Archive', 'BaseArchive', 'ASCENDING', 'DESCENDING', 'OUTGOING', 'INCOMING'
 
 IdT = TypeVar('IdT')  # The archive ID type
 
@@ -13,6 +17,9 @@ IdT = TypeVar('IdT')  # The archive ID type
 ASCENDING = 1
 DESCENDING = -1
 
+OUTGOING = 1
+INCOMING = -1
+
 
 class Archive(Generic[IdT], metaclass=abc.ABCMeta):
     """An archive provides the persistent storage for the historian.  It is responsible for storing,
@@ -20,8 +27,7 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
 
     # pylint: disable=too-many-public-methods
 
-    RefEdge = NamedTuple('RefEdge', [('source', SnapshotRef[IdT]), ('target', SnapshotRef[IdT])])
-    RefGraph = Iterable[RefEdge]
+    SnapshotId = records.SnapshotId[IdT]
     MetaEntry = NamedTuple('MetaEntry', [('obj_id', IdT), ['meta', dict]])
 
     @classmethod
@@ -56,9 +62,13 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
         """Save a data record to the archive"""
 
     @abc.abstractmethod
-    def save_many(self, records: Sequence[DataRecord]):
+    def save_many(self, data_records: Sequence[DataRecord]):
         """Save many data records to the archive"""
 
+    @abc.abstractmethod
+    def bulk_write(self, ops: Sequence[operations.Operation]):
+        """Made a collection of write operations to the database"""
+
     # region Metadata
 
     @abc.abstractmethod
@@ -76,8 +86,8 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def meta_set_many(self, metas: Mapping[IdT, Optional[Mapping]]):
-        """Set the metadata on multiple objects.  This method expects to get multiple tuples
-        containing the object id and corresponding metadata"""
+        """Set the metadata on multiple objects.  This takes a mapping of the object id to the
+        corresponding (optional) metadata dictionary"""
 
     @abc.abstractmethod
     def meta_update(self, obj_id: IdT, meta: Mapping):
@@ -91,7 +101,7 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
     @abc.abstractmethod
     def meta_find(
             self,
-            filter: dict,  # pylint: disable=redefined-builtin
+            filter: dict = None,  # pylint: disable=redefined-builtin
             obj_id: Union[IdT, Iterable[IdT], Mapping] = None) -> \
             'Iterator[Archive.MetaEntry]':
         """Yield metadata satisfying the given criteria.  The search can optionally be restricted to
@@ -104,6 +114,26 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
             3. a general query filter to be applied to the object ids
         """
 
+    @abc.abstractmethod
+    def meta_distinct(
+            self,
+            key: str,
+            filter: dict = None,  # pylint: disable=redefined-builtin
+            obj_id: Union[IdT, Iterable[IdT], Mapping] = None) -> \
+            'Iterator':
+        """Yield distinct values found for 'key' within metadata documents, optionally marching a
+        search filter.
+
+        The search can optionally be restricted to a set of passed object ids.
+
+        :param key: the document key to get distinct values for
+        :param filter: a query filter for the search
+        :param obj_id: an optional restriction on the object ids to search.  This ben be either:
+            1. a single object id
+            2. an iterable of object ids in which is treated as {'$in': list(obj_ids)}
+            3. a general query filter to be applied to the object ids
+        """
+
     @abc.abstractmethod
     def meta_create_index(self, keys, unique=False, where_exist=False):
         """Create an index on the metadata.  Takes either a single key or list of (key, direction)
@@ -117,7 +147,7 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
     # endregion
 
     @abc.abstractmethod
-    def load(self, reference: SnapshotRef[IdT]) -> DataRecord:
+    def load(self, snapshot_id: SnapshotId) -> DataRecord:
         """Load a snapshot of an object with the given reference"""
 
     @abc.abstractmethod
@@ -126,32 +156,36 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
         records"""
 
     @abc.abstractmethod
-    def get_snapshot_refs(self, obj_id: IdT) -> Sequence[SnapshotRef[IdT]]:
-        """Returns a list of time ordered snapshot references"""
+    def get_snapshot_ids(self, obj_id: IdT) -> 'Sequence[Archive.SnapshotId]':
+        """Returns a list of time ordered snapshot ids"""
 
     # pylint: disable=too-many-arguments
     @abc.abstractmethod
     def find(self,
              obj_id: Union[IdT, Iterable[IdT], Dict] = None,
              type_id=None,
-             created_by=None,
-             copied_from=None,
-             version=None,
+             created_by: Optional[IdT] = None,
+             copied_from: Optional[IdT] = None,
+             version: int = None,
              state=None,
+             state_types=None,
              snapshot_hash=None,
-             meta=None,
+             meta: dict = None,
+             extras: dict = None,
              limit=0,
              sort=None,
-             skip=0):
+             skip=0) -> Iterator[DataRecord]:
         """Find records matching the given criteria
 
-        :param type_id: the type id to look for
-        :param created_by: find records with the given type id
+        :param type_id: find records with the given type id
+        :param created_by: find records with the given created by id
         :param copied_from: find records copied from the record with the given id
         :param version: restrict the search to this version, -1 for latest
         :param state: find objects with this state filter
+        :param state_types: file objects with this state types filter
         :param snapshot_hash: find objects with this snapshot hash
         :param meta: find objects with this meta filter
+        :param extras: the search criteria to apply on the data record extras
         :param limit: limit the results to this many records
         :param obj_id: an optional restriction on the object ids to search.  This ben be either:
             1. a single object id
@@ -161,6 +195,15 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
         :param skip: skip the this many entries
         """
 
+    @abc.abstractmethod
+    def distinct(self, key: str, filter: dict = None) -> Iterator:  # pylint: disable=redefined-builtin
+        """Get distinct values of the given record key
+
+        :param key: the key to find distinct values for, see DataRecord for possible keys
+        :param filter: an optional filter to restrict the search to.  Should be a dictionary that
+            filters on entries in the DataRecord i.e. the kwargs that can be passed to find().
+        """
+
     @abc.abstractmethod
     def count(self,
               obj_id=None,
@@ -175,9 +218,23 @@ class Archive(Generic[IdT], metaclass=abc.ABCMeta):
         """Count the number of entries that match the given query"""
 
     @abc.abstractmethod
-    def get_reference_graph(self,
-                            srefs: Sequence[SnapshotRef[IdT]]) -> 'Sequence[Archive.RefGraph]':
-        """Given one or more object ids the archive will supply the corresponding reference graph(s)
+    def get_snapshot_ref_graph(self,
+                               *snapshot_ids: SnapshotId,
+                               direction=OUTGOING,
+                               max_dist: int = None) -> networkx.DiGraph:
+        """Given one or more snapshot ids the archive will supply the corresponding reference
+        graph(s).  The graphs start at the given id and contains all snapshots that it references,
+        all snapshots they reference and so on.
+        """
+
+    @abc.abstractmethod
+    def get_obj_ref_graph(self,
+                          *obj_ids: IdT,
+                          direction=OUTGOING,
+                          max_dist: int = None) -> networkx.DiGraph:
+        """Given one or more object ids the archive will supply the corresponding reference
+        graph(s).  The graphs start at the given id and contains all object ids that it references,
+        all object ids they reference and so on.
         """
 
 
@@ -189,13 +246,15 @@ class BaseArchive(Archive[IdT]):
         assert cls.ID_TYPE, "The ID type has not been set on this archive"
         return cls.ID_TYPE
 
-    def save_many(self, records: Sequence[DataRecord]):
+    def save(self, record: DataRecord):
+        return self.bulk_write([operations.Insert(record)])
+
+    def save_many(self, data_records: Sequence[DataRecord]):
         """
         This will save records one by one but subclass may want to override this behaviour if
         they can save multiple records at once.
         """
-        for record in records:
-            self.save(record)
+        self.bulk_write([operations.Insert(record) for record in data_records])
 
     def meta_get_many(self, obj_ids: Iterable[IdT]) -> Dict[IdT, dict]:
         metas = {}
@@ -212,7 +271,7 @@ class BaseArchive(Archive[IdT]):
             self.meta_set(*entry)
 
     def history(self, obj_id: IdT, idx_or_slice) -> [DataRecord, Sequence[DataRecord]]:
-        refs = self.get_snapshot_refs(obj_id)[idx_or_slice]
+        refs = self.get_snapshot_ids(obj_id)[idx_or_slice]
         if len(refs) > 1:
             return [self.load(ref) for ref in refs]
 
@@ -233,7 +292,7 @@ def scalar_query_spec(specifier: Union[Mapping, Iterable[Any], Any]) -> \
         {'$in': list(iterable)}
     3. it is a raw item item in which case it is matched directly
     """
-    if isinstance(specifier, Mapping):
+    if isinstance(specifier, dict):  # This has to be first as dict is iterable
         return specifier
     if isinstance(specifier, Iterable):
         return q.in_(*specifier)
