@@ -9,11 +9,14 @@ from . import types
 
 __all__ = 'BaseSavableObject', 'ConvenienceMixin', 'SimpleSavable', 'AsRef'
 
-AsRef = collections.namedtuple('AsRef', 'attr')
-
 AttrSpec = collections.namedtuple('AttrSpec', 'name as_ref')
 
 
+def AsRef(name: str) -> AttrSpec:  # pylint: disable=invalid-name
+    """Create an attribute specification for an attribute that should be stored by reference"""
+    return AttrSpec(name, True)
+
+
 class BaseSavableObject(types.SavableObject):
     """A helper class that makes a class compatible with the historian by flagging certain
     attributes which will be saved/loaded/hashed and compared in __eq__.  This should be an
@@ -25,18 +28,21 @@ class BaseSavableObject(types.SavableObject):
     def __new__(cls, *_args, **_kwargs):
         new_instance = super(BaseSavableObject, cls).__new__(cls)
         attrs = {}
-        for entry in cls.__mro__:
+        for entry in cls.mro():
             try:
-                for name in getattr(entry, 'ATTRS'):
-                    if name not in attrs:
-                        if isinstance(name, AsRef):
-                            spec = AttrSpec(name.attr, True)
-                        else:
-                            spec = AttrSpec(name, False)
-                        attrs[name] = spec
-
+                class_attrs = getattr(entry, 'ATTRS')
             except AttributeError:
                 pass
+            else:
+                for attr_spec in class_attrs:
+                    if isinstance(attr_spec, str):
+                        # If it's just a string then default to store by value
+                        attr_spec = AttrSpec(attr_spec, False)
+
+                    # Check that it's not already there so higher up in the MRO is always kept
+                    if attr_spec.name not in attrs:
+                        attrs[attr_spec.name] = attr_spec
+
         setattr(new_instance, '__attrs', attrs.values())
         return new_instance
 
@@ -74,7 +80,10 @@ class BaseSavableObject(types.SavableObject):
                     raise
             else:
                 if attr.as_ref:
-                    obj = obj()
+                    if obj:
+                        obj = obj()
+                    else:
+                        obj = None
                 setattr(self, attr.name, obj)
 
     def __get_attrs(self) -> typing.Sequence[AttrSpec]:
@@ -119,7 +128,8 @@ class ConvenienceMixin:
 
     def save(self, meta: dict = None):
         """Save the object"""
-        return mincepy.get_historian().save_one(self, meta=meta)
+        historian = self._historian or mincepy.get_historian()
+        return historian.save_one(self, meta=meta)
 
     def sync(self):
         """Update the state of this object by loading the latest version from the historian"""
@@ -130,7 +140,7 @@ class ConvenienceMixin:
         self._on_save(saver)
         return super().save_instance_state(saver)
 
-    def load_instance_state(self, saved_state, loader: depositors.Loader):
+    def load_instance_state(self, saved_state, loader: 'mincepy.Loader'):
         """Take the given object and load the instance state into it"""
         super().load_instance_state(saved_state, loader)
         self._on_load(loader)
