@@ -1,12 +1,19 @@
 from collections import namedtuple
 import contextlib
-import copy
+
+try:
+    from contextlib import nullcontext
+except ImportError:
+    from contextlib2 import nullcontext
 import getpass
 import logging
 import socket
 import typing
-from typing import MutableMapping, Any, Optional, Mapping, Iterable, Union, Iterator, Dict
+from typing import MutableMapping, Any, Optional, Iterable, Union, Iterator, Type
 import weakref
+import warnings
+
+import deprecation
 
 from . import archives
 from . import builtins
@@ -15,11 +22,16 @@ from . import depositors
 from . import refs
 from . import exceptions
 from . import helpers
-from . import process
+from . import hist
+from . import migrate
+from . import operations
 from . import records
+from . import staging
+from . import tracking
 from . import types
 from . import type_registry
 from . import utils
+from . import version as version_mod
 from .transactions import RollbackTransaction, Transaction, LiveObjects
 
 __all__ = 'Historian', 'ObjectEntry'
@@ -30,133 +42,20 @@ ObjectEntry = namedtuple('ObjectEntry', 'ref obj')
 HistorianType = Union[helpers.TypeHelper, typing.Type[types.SavableObject]]
 
 
-class Meta:
-    """A class for grouping metadata related methods"""
-
-    # Meta is a 'friend' of Historian and so can access privates pylint: disable=protected-access
-
-    def __init__(self, historian):
-        self._hist = historian  # type: 'Historian'
-        self._sticky = {}
-
-    @property
-    def sticky(self) -> dict:
-        return self._sticky
-
-    def get(self, obj_or_identifier) -> Optional[dict]:
-        """Get the metadata for an object
-
-        :param obj_or_identifier: either the object instance, an object ID or a snapshot reference
-        """
-        results = self.get_many((obj_or_identifier,))
-        assert len(results) == 1
-        meta = tuple(results.values())[0]
-        return meta
-
-    def get_many(self, obj_or_identifiers) -> Dict[Any, dict]:
-        obj_ids = set(
-            self._hist._ensure_obj_id(obj_or_identifier)
-            for obj_or_identifier in obj_or_identifiers)
-        trans = self._hist.current_transaction()
-        if trans:
-            # First, get what we can from the archive
-            found = {}
-            for obj_id in obj_ids:
-                try:
-                    found[obj_id] = trans.get_meta(obj_id)
-                except exceptions.NotFound:
-                    pass
-
-            # Now get anything else from the archive
-            obj_ids -= found.keys()
-            if obj_ids:
-                from_archive = self._hist.archive.meta_get_many(obj_ids)
-                # Now put into the transaction so it doesn't look it up again.
-                for obj_id in obj_ids:
-                    trans.set_meta(obj_id, from_archive[obj_id])
-                found.update(from_archive)
-
-            return found
-
-        # No transaction
-        return self._hist.archive.meta_get_many(obj_ids)
-
-    def set(self, obj_or_identifier, meta: Optional[Mapping]):
-        """Set the metadata for an object
-
-        :param obj_or_identifier: either the object instance, an object ID or a snapshot reference
-        :param meta: the metadata dictionary
-        """
-        obj_id = self._hist._ensure_obj_id(obj_or_identifier)
-        trans = self._hist.current_transaction()
-        if trans:
-            return trans.set_meta(obj_id, meta)
-
-        return self._hist.archive.meta_set(obj_id, meta)
-
-    def set_many(self, metas: Mapping[Any, Optional[dict]]):
-        mapped = {self._hist._ensure_obj_id(ident): meta for ident, meta in metas.items()}
-        trans = self._hist.current_transaction()
-        if trans:
-            for entry in mapped.items():
-                trans.set_meta(*entry)
-        else:
-            self._hist.archive.meta_set_many(mapped)
-
-    def update(self, obj_or_identifier, meta: Mapping):
-        """Update the metadata for an object
-
-        :param obj_or_identifier: either the object instance, an object ID or a snapshot reference
-        :param meta: the metadata dictionary
-        """
-        obj_id = self._hist._ensure_obj_id(obj_or_identifier)
-        trans = self._hist.current_transaction()
-        if trans:
-            # Update the metadata in the transaction
-            try:
-                current = trans.get_meta(obj_id)
-            except exceptions.NotFound:
-                current = self._hist.archive.meta_get(obj_id)  # Try the archive
-                if current is None:
-                    current = {}  # Ok, no meta
-
-            current.update(meta)
-            trans.set_meta(obj_id, current)
-        else:
-            self._hist.archive.meta_update(obj_id, meta)
-
-    def update_many(self, metas: Mapping[Any, Optional[dict]]):
-        mapped = {self._hist._ensure_obj_id(ident): meta for ident, meta in metas.items()}
-        trans = self._hist.current_transaction()
-        if trans:
-            for entry in mapped.items():
-                self.update(*entry)
-        else:
-            self._hist.archive.meta_update_many(mapped)
-
-    def find(self, filter, obj_id=None):  # pylint: disable=redefined-builtin
-        """Find metadata matching the given criteria.  Ever returned metadata dictionary will
-        contain an 'obj_id' key which identifies the object it belongs to"""
-        return self._hist.archive.meta_find(filter=filter, obj_id=obj_id)
-
-    def create_index(self, keys, unique=False, where_exist=False):
-        """Create an index on the metadata.  Takes either a single key or list of (key, direction)
-         pairs
-
-         :param keys: the key or keys to create the index on
-         :param unique: if True, create a uniqueness constraint on this index
-         :param where_exist: if True, only apply this index on documents that contain the key(s)
-         """
-        self._hist.archive.meta_create_index(keys, unique=unique, where_exist=where_exist)
-
-
-class Historian:
+class Historian:  # pylint: disable=too-many-public-methods, too-many-instance-attributes
     """The historian acts as a go-between between your python objects and the archive which is
     a persistent store of the records.  It will keep track of all live objects (i.e. those that
     have active references to them) that have been loaded and/or saved as well as enabling the
     user to lookup objects in the archive."""
 
-    # pylint: disable=too-many-public-methods
+    @deprecation.deprecated(deprecated_in="0.14.5",
+                            removed_in="0.16",
+                            current_version=version_mod.__version__,
+                            details="Use mincepy.copy() instead")
+    def copy(self, obj):  # pylint: disable=no-self-use
+        """Create a shallow copy of the object.  Using this method allows the historian to inject
+        information about where the object was copied from into the record if saved."""
+        return tracking.copy(obj)
 
     def __init__(self, archive: archives.Archive, equators=()):
         self._archive = archive
@@ -164,16 +63,14 @@ class Historian:
         # Register default types
         self._type_registry = type_registry.TypeRegistry()
         self.register_type(refs.ObjRef)
-        self.register_type(helpers.SnapshotRefHelper())
+        self.register_type(builtins.SnapshotIdHelper())
         self.register_types(archive.get_types())
 
         # Snapshot objects -> reference. Objects that were loaded from historical snapshots
-        self._snapshots_objects = utils.WeakObjectIdDict()  # type: MutableMapping[Any, records.Ref]
+        self._snapshots_objects = utils.WeakObjectIdDict(
+        )  # type: MutableMapping[Any, records.SnapshotId]
         self._live_objects = LiveObjects()
 
-        # Staged objects that have been created but not saved
-        self._type_ids = {}
-
         self._transactions = None
 
         self._saving_set = set()
@@ -181,17 +78,35 @@ class Historian:
         self._user = getpass.getuser()
         self._hostname = socket.gethostname()
 
-        self._meta = Meta(self)
+        self._live_depositor = depositors.LiveDepositor(self)
+        self._meta = hist.Meta(self, self._archive)
+        self._migrate = migrate.Migrations(self)
+        self._references = hist.References(self)
 
     @property
     def archive(self):
         return self._archive
 
+    @property
+    def meta(self) -> hist.Meta:
+        """Access to functions that operate on the metadata"""
+        return self._meta
+
     @property
     def primitives(self) -> tuple:
         """A tuple of all the primitive types"""
         return types.PRIMITIVE_TYPES + (self._archive.get_id_type(),)
 
+    @property
+    def migrations(self) -> migrate.Migrations:
+        """Access the migration possibilities"""
+        return self._migrate
+
+    @property
+    def references(self) -> hist.References:
+        """Access the references possibilities"""
+        return self._references
+
     def create_file(self, filename: str = None, encoding: str = None) -> builtins.BaseFile:
         """Create a new file.  The historian will supply file type compatible with the archive in
          use."""
@@ -216,7 +131,7 @@ class Historian:
             to_save.append(entry)
 
         ids = []
-        with self.transaction():
+        with self.in_transaction():
             for entry in to_save:
                 ids.append(self.save_one(*entry))
 
@@ -241,8 +156,8 @@ class Historian:
             raise TypeError("Metadata must be a dictionary, got type '{}'".format(type(meta)))
 
         # Save the object and metadata
-        with self.transaction():
-            record = self._save_object(obj, depositors.LiveDepositor(self))
+        with self.in_transaction():
+            record = self._save_object(obj, self._live_depositor)
             if meta:
                 self.meta.update(record.obj_id, meta)
 
@@ -262,10 +177,11 @@ class Historian:
         to continue the history of the object as the original rather than a brand new object.  Then
         just replace the old object with the new one by calling this function.
         """
-        assert not self.current_transaction(
-        ), "Can't replace during a transaction for the time being"
-        assert isinstance(new, type(old)), "Can't replace type '{} with type '{}!".format(
-            type(old), type(new))
+        if self.current_transaction() is not None:
+            raise RuntimeError("Can't replace during a transaction for the time being")
+
+        if not isinstance(new, type(old)):
+            raise TypeError("Can't replace type '{} with type '{}!".format(type(old), type(new)))
 
         # Get the current record and replace the object with the new one
         record = self._live_objects.get_record(old)
@@ -273,10 +189,10 @@ class Historian:
         self._live_objects.insert(new, record)
 
         # Make sure creators is correct as well
-        process.CreatorsRegistry.set_creator(new, process.CreatorsRegistry.get_creator(old))
+        staging.replace(old, new)
 
-    def load_snapshot(self, reference: records.SnapshotRef) -> Any:
-        return depositors.SnapshotLoader(self).load(reference)
+    def load_snapshot(self, snapshot_id: records.SnapshotId) -> Any:
+        return self._new_snapshot_depositor().load(snapshot_id)
 
     def load(self, *obj_ids_or_refs):
         """Load object(s) or snapshot(s)."""
@@ -291,12 +207,12 @@ class Historian:
 
     def load_one(self, obj_id_or_ref):
         """Load one object or shot from the database"""
-        if isinstance(obj_id_or_ref, records.SnapshotRef):
+        if isinstance(obj_id_or_ref, records.SnapshotId):
             return self.load_snapshot(obj_id_or_ref)
 
-        return self._load_object(obj_id_or_ref, depositors.LiveDepositor(self))
+        return self._load_object(obj_id_or_ref, self._live_depositor)
 
-    def sync(self, obj) -> bool:
+    def sync(self, obj: object) -> bool:
         """Update an object with the latest state in the database.
         If there is no new version in the archive then the current version remains
         unchanged including any modifications.
@@ -308,35 +224,22 @@ class Historian:
             # Never saved so the object is as up to date as can be!
             return False
 
-        ref = self._get_latest_snapshot_reference(obj_id)
-        archive_record = self._archive.load(ref)
-        if archive_record.is_deleted_record():
+        sid = self._get_latest_snapshot_reference(obj_id)
+
+        try:
+            record = next(self._archive.find(obj_id=obj_id, version=-1))
+        except StopIteration:
+            raise exceptions.NotFound(obj_id)
+
+        if record.is_deleted_record():
             raise exceptions.ObjectDeleted("Object with id '{}' has been deleted".format(obj_id))
 
-        if ref.version == self.get_snapshot_ref(obj).version:
+        if sid.version == self.get_snapshot_id(obj).version:
             # Nothing has changed
             return False
 
         # The one in the archive is newer, so use that
-        depositor = depositors.LiveDepositor(self)
-        return depositor.update_from_record(obj, archive_record)
-
-    def copy(self, obj):
-        """Create a shallow copy of the object, save that copy and return it"""
-        with self.transaction() as trans:
-            record = self._save_object(obj, depositors.LiveDepositor(self))
-            copy_builder = record.copy_builder(obj_id=self._archive.create_archive_id())
-            self._record_builder_created(copy_builder)
-
-            # Copy the object and record
-            obj_copy = copy.copy(obj)
-            obj_copy_record = copy_builder.build()
-
-            # Insert all the new objects into the transaction
-            trans.insert_live_object(obj_copy, obj_copy_record)
-            trans.stage(obj_copy_record)
-
-        return obj_copy
+        return self._live_depositor.update_from_record(obj, record)
 
     def delete(self, obj_or_identifier):
         """Delete an object.
@@ -361,27 +264,27 @@ class Historian:
                 # not even in the archive
                 raise exc
 
-        with self.transaction() as trans:
+        with self.in_transaction() as trans:
             builder = records.make_deleted_builder(record)
             deleted_record = self._record_builder_created(builder).build()
             trans.delete(obj_id)
-            trans.stage(deleted_record)
+            trans.stage(operations.Insert(deleted_record))
 
     def history(
             self,
             obj_or_obj_id,
             idx_or_slice='*',
             as_objects=True) -> [typing.Sequence[ObjectEntry], typing.Sequence[records.DataRecord]]:
-        """
-        Get a sequence of object ids and instances from the history of the given object.
+        """Get a sequence of object ids and instances from the history of the given object.
 
         :param obj_or_obj_id: The instance or id of the object to get the history for
         :param idx_or_slice: The particular index or a slice of which historical versions to get
         :param as_objects: if True return the object instances, otherwise returns the DataRecords
 
         Example:
-        >>> historian = get_historian()
-        >>> car = Car('ferrari', 'white')
+        >>> import mincepy, mincepy.testing
+        >>> historian = mincepy.get_historian()
+        >>> car = mincepy.testing.Car('ferrari', 'white')
         >>> car_id = historian.save(car)
         >>> car.colour = 'red'
         >>> historian.save(car)
@@ -395,7 +298,7 @@ class Historian:
         >>> history[1].obj is car
         """
         obj_id = self._ensure_obj_id(obj_or_obj_id)
-        snapshot_refs = self._archive.get_snapshot_refs(obj_id)
+        snapshot_refs = self._archive.get_snapshot_ids(obj_id)
         indices = utils.to_slice(idx_or_slice)
         to_get = snapshot_refs[indices]
         if as_objects:
@@ -403,11 +306,6 @@ class Historian:
 
         return [self._archive.load(ref) for ref in to_get]
 
-    @property
-    def meta(self) -> Meta:
-        """Access to functions that operate on the metadata"""
-        return self._meta
-
     def get_current_record(self, obj) -> records.DataRecord:
         """Get a record for an object known to the historian"""
         trans = self.current_transaction()
@@ -432,7 +330,7 @@ class Historian:
         trans = self.current_transaction()
         if trans is not None:
             try:
-                return trans.get_reference_for_live_object(obj).obj_id
+                return trans.get_snapshot_id_for_live_object(obj).obj_id
             except exceptions.NotFound:
                 pass
 
@@ -497,16 +395,17 @@ class Historian:
         # Out of options
         return None
 
-    def get_snapshot_ref(self, obj):
-        """Get the current snapshot reference for a live object"""
+    def get_snapshot_id(self, obj) -> records.SnapshotId:
+        """Get the current snapshot id for a live object.  Will return the id or raise
+        :class:`mincepy.NotFound` exception"""
         trans = self.current_transaction()
         if trans:
             try:
-                return trans.get_reference_for_live_object(obj)
+                return trans.get_snapshot_id_for_live_object(obj)
             except exceptions.NotFound:
                 pass
 
-        return self._live_objects.get_record(obj).get_reference()
+        return self._live_objects.get_record(obj).snapshot_id
 
     def hash(self, obj):
         return self._equator.hash(obj)
@@ -528,6 +427,10 @@ class Historian:
         except TypeError:
             return False
 
+    @property
+    def type_registry(self) -> type_registry.TypeRegistry:
+        return self._type_registry
+
     def is_primitive(self, obj) -> bool:
         """Check if the object is one of the primitives and should be saved by value in the
         archive"""
@@ -579,7 +482,7 @@ class Historian:
         be necessary to look at the details of the `save_instance_state()` method for that type.
         Metadata is always a dictionary containing primitives (strings, dicts, lists, etc).
 
-        For the most part, the filter syntax of `mincepy` conforms to that of `MongoDB`_ with
+        For the most part, the filter syntax of `mincePy` conforms to that of `MongoDB`_ with
         convenience functions locate in :py:mod:`mincepy.qops` that can make it easier to
         to build a query.
 
@@ -588,6 +491,7 @@ class Historian:
         Find all :py:class:`~mincepy.testing.Car`s that are brown or red:
 
         >>> import mincepy
+        >>> historian = mincepy.get_historian()
         >>> historian.find(state=dict(colour=mincepy.q.in_('brown', 'red')))
 
         Find all people that are older than 34 and live in Edinburgh:
@@ -605,16 +509,29 @@ class Historian:
         :param skip: the page to get results from
         """
         # pylint: disable=too-many-arguments
-        results = self.find_records(obj_type=obj_type,
-                                    obj_id=obj_id,
-                                    version=version,
-                                    state=state,
-                                    meta=meta,
-                                    sort=sort,
-                                    limit=limit,
-                                    skip=skip)
-        for result in results:
-            yield self.load(result.obj_id)
+        if version != -1:
+            warnings.warn("The find() kwarg 'version' will be deprecated in version 0.16")
+
+        if not sort and not skip and not meta and not limit:
+            # Optimisation: get the object ids from distinct to avoid loading the records until
+            # the load() call
+            for oid in self.find_distinct(records.OBJ_ID,
+                                          obj_type=obj_type,
+                                          obj_id=obj_id,
+                                          version=version,
+                                          state=state):
+                yield self.load(oid)
+        else:
+            results = self.find_records(obj_type=obj_type,
+                                        obj_id=obj_id,
+                                        version=version,
+                                        state=state,
+                                        meta=meta,
+                                        sort=sort,
+                                        limit=limit,
+                                        skip=skip)
+            for result in results:
+                yield self.load(result.obj_id)
 
     def find_records(self,
                      obj_type=None,
@@ -622,6 +539,7 @@ class Historian:
                      version: int = -1,
                      state=None,
                      meta: dict = None,
+                     extras: dict = None,
                      sort=None,
                      limit=0,
                      skip=0) -> Iterator[records.DataRecord]:
@@ -633,6 +551,7 @@ class Historian:
         :param state: the criteria on the state of the object to apply
         :type state: must be subclass of historian.primitive
         :param meta: the search criteria to apply on the metadata of the object
+        :param extras: the search criteria to apply on the data record extras
         :param sort: the sort criteria
         :param limit: the maximum number of results to return, 0 means unlimited
         :param skip: the page to get results from
@@ -644,24 +563,72 @@ class Historian:
                 type_id = self.get_obj_type_id(obj_type)
             except TypeError:
                 pass
+
+        if obj_id is not None:
+            # Convert object ids to the expected type before passing to archive
+            if isinstance(obj_id, list):
+                obj_id = list(self._ensure_obj_id(oid) for oid in obj_id)
+            else:
+                obj_id = self._ensure_obj_id(obj_id)
+
         results = self._archive.find(obj_id=obj_id,
                                      type_id=type_id,
                                      state=state,
                                      version=version,
                                      meta=meta,
+                                     extras=extras,
                                      sort=sort,
                                      limit=limit,
                                      skip=skip)
         yield from results
 
+    # pylint: disable=too-many-arguments
+    def find_distinct(self,
+                      key: str,
+                      obj_type=None,
+                      obj_id=None,
+                      version: int = -1,
+                      state=None,
+                      extras: dict = None):
+        """Get distinct values of the given record key
+
+        :param key: the key to find distinct values for, see DataRecord for possible keys
+        :param version: convenience for setting the version to search for.  Can also be specified in
+            the filter in which case this will be ignored.
+        """
+        record_filter = {}
+        if obj_type is not None:
+            if obj_type is not None:
+                try:
+                    type_id = self.get_obj_type_id(obj_type)
+                except TypeError:
+                    type_id = obj_type
+
+            record_filter[records.TYPE_ID] = type_id
+        if obj_id is not None:
+            # Convert object ids to the expected type before passing to archive
+            if isinstance(obj_id, list):
+                obj_id = list(self._ensure_obj_id(oid) for oid in obj_id)
+            else:
+                obj_id = self._ensure_obj_id(obj_id)
+            record_filter[records.OBJ_ID] = obj_id
+        if version is not None:
+            record_filter[records.VERSION] = version
+        if state is not None:
+            record_filter[records.STATE] = state
+        if extras is not None:
+            record_filter[records.EXTRAS] = extras
+
+        yield from self._archive.distinct(key, record_filter)
+
     def get_creator(self, obj_or_identifier):
         """Get the object that created the passed object"""
         if not self.is_obj_id(obj_or_identifier):
-            # Object instance, try our creators cache
-            try:
-                return process.CreatorsRegistry.get_creator(obj_or_identifier)
-            except KeyError:
-                pass
+            # Object instance, try the staging area
+            info = staging.get_info(obj_or_identifier, create=False) or {}
+            created_by = info.get(records.ExtraKeys.CREATED_BY, None)
+            if created_by is not None:
+                return created_by
 
         creator_id = self.created_by(obj_or_identifier)
         return self.load_one(creator_id)
@@ -673,11 +640,16 @@ class Historian:
         except exceptions.NotFound:
             if not self.is_obj_id(obj_or_identifier):
                 raise
-            record = self._archive.load(self._get_latest_snapshot_reference(obj_or_identifier))
+
+            try:
+                record = next(self._archive.find(obj_id=obj_or_identifier, version=-1))
+            except StopIteration:
+                raise exceptions.NotFound(obj_or_identifier)
 
         return record.created_by
 
     def get_user_info(self) -> dict:
+        """Get information about the current user and host"""
         user_info = {}
         if self._user:
             user_info[records.ExtraKeys.USER] = self._user
@@ -685,6 +657,19 @@ class Historian:
             user_info[records.ExtraKeys.HOSTNAME] = self._hostname
         return user_info
 
+    @contextlib.contextmanager
+    def in_transaction(self):
+        """This context will either re-use an existing transaction, if one is currently taking place
+        or create a new one if not."""
+        current = self.current_transaction()
+        if current is None:
+            ctx = self.transaction()
+        else:
+            ctx = nullcontext(current)
+
+        with ctx as trans:
+            yield trans
+
     @contextlib.contextmanager
     def transaction(self):
         """Start a new transaction.  Will be nested if there is already one underway"""
@@ -694,8 +679,11 @@ class Historian:
                 self._transactions.append(nested)
                 try:
                     yield nested
-                finally:
+                except Exception:  # Need this so we can have 'else' pylint: disable=try-except-raise
+                    raise
+                else:
                     self._closing_transaction(nested)
+                finally:
                     popped = self._transactions.pop()
                     assert popped is nested
         else:
@@ -722,7 +710,21 @@ class Historian:
         return self._transactions[-1]
 
     def _closing_transaction(self, trans: Transaction):
-        pass
+        conflicting = set()
+
+        # Filter out the deleted records
+        del_ops = filter(
+            lambda _: isinstance(_, operations.Insert) and _.record.is_deleted_record(),
+            trans.staged)
+
+        obj_ids = set(operation.obj_id for operation in del_ops)
+        ref_graph = self.references.get_obj_ref_graph(*obj_ids, direction=archives.INCOMING)
+        for obj_id in obj_ids:
+            for edge in ref_graph.in_edges(obj_id):
+                conflicting.add(edge[1])
+
+        if conflicting:
+            raise exceptions.ReferenceError("Cannot perform delete", conflicting)
 
     def _commit_transaction(self, trans: Transaction):
         """Commit a transaction that is finishing"""
@@ -741,23 +743,23 @@ class Historian:
 
         # Save any records that were staged for archiving
         if trans.staged:
-            self._archive.save_many(trans.staged)
+            self._archive.bulk_write(trans.staged)
 
         # Metas
-        for obj_id, meta in trans.metas.items():
-            self._archive.meta_set(obj_id, meta)
+        if trans.metas:
+            self._archive.meta_set_many(trans.metas)
 
-    def _get_latest_snapshot_reference(self, obj_id) -> records.SnapshotRef:
+    def _get_latest_snapshot_reference(self, obj_id) -> records.SnapshotId:
         """Given an object id this will return a reference to the latest snapshot"""
         try:
-            return self._archive.get_snapshot_refs(obj_id)[-1]
+            return self._archive.get_snapshot_ids(obj_id)[-1]
         except IndexError:
             raise exceptions.NotFound("Object with id '{}' not found.".format(obj_id))
 
     def _load_object(self, obj_id, depositor: depositors.LiveDepositor):
         obj_id = self._ensure_obj_id(obj_id)
 
-        with self.transaction() as trans:
+        with self.in_transaction() as trans:
             # Try getting the object from the our dict of up to date ones
             try:
                 return trans.get_live_object(obj_id)
@@ -779,17 +781,19 @@ class Historian:
             try:
                 obj = self._live_objects.get_object(obj_id)
             except exceptions.NotFound:
+                logger.debug("Loading object from record: %s", record.snapshot_id)
                 # Ok, just use the one from the archive
                 return depositor.load_from_record(record)
             else:
-                if record.version != self.get_snapshot_ref(obj).version:
+                if record.version != self.get_snapshot_id(obj).version:
                     # The one in the archive is newer, so use that
+                    logger.debug("Updating object from record: %s", record.snapshot_id)
                     depositor.update_from_record(obj, record)
 
                 return obj
 
     def _save_object(self, obj, depositor) -> records.DataRecord:
-        with self.transaction() as trans:
+        with self.in_transaction() as trans:
             try:
                 helper = self._ensure_compatible(type(obj))
             except TypeError:
@@ -811,11 +815,11 @@ class Historian:
                     record = self._live_objects.get_record(obj)
                 except exceptions.NotFound:
                     # Object being saved for the first time
-                    builder = self._create_builder(obj, dict(snapshot_hash=current_hash))
+                    builder = self._create_builder(obj, snapshot_hash=current_hash)
                     record = depositor.save_from_builder(obj, builder)
                     if self.meta.sticky:
                         # Apply the sticky meta
-                        trans.set_meta(record.obj_id, self.meta.sticky)
+                        self.meta.update(record.obj_id, self.meta.sticky)
                     return record
                 else:
                     if helper.IMMUTABLE:
@@ -824,26 +828,26 @@ class Historian:
                         return record
 
                     # Check if our record is up to date
-                    with self.transaction() as transaction:
-                        loaded_obj = depositors.SnapshotLoader(self).load_from_record(record)
+                    with self.transaction() as nested:
+                        loaded_obj = self._new_snapshot_depositor().load_from_record(record)
 
                         if current_hash == record.snapshot_hash and self.eq(obj, loaded_obj):
                             # Objects identical
-                            transaction.rollback()
+                            nested.rollback()
                         else:
                             builder = record.child_builder(snapshot_hash=current_hash)
-                            self._record_builder_created(builder)
                             record = depositor.save_from_builder(obj, builder)
 
                     return record
 
-    def _create_builder(self, obj, additional=None):
+    def _create_builder(self, obj, **additional) -> records.DataRecordBuilder:
+        """Create a record builder for a new object object"""
         additional = additional or {}
         helper = self._ensure_compatible(type(obj))
+
         builder = records.DataRecord.new_builder(type_id=helper.TYPE_ID,
                                                  obj_id=self._archive.create_archive_id(),
                                                  version=0)
-        self._record_builder_created(builder)
         builder.update(additional)
         return builder
 
@@ -863,9 +867,9 @@ class Historian:
 
         return obj_id
 
-    def _ensure_compatible(self, obj_type) -> helpers.TypeHelper:
+    def _ensure_compatible(self, obj_type: Type[types.SavableObject]) -> helpers.TypeHelper:
         if obj_type not in self._type_registry:
-            return self.register_type(obj_type)
+            self.register_type(obj_type)
 
         return self._type_registry.get_helper_from_obj_type(obj_type)
 
@@ -887,3 +891,6 @@ class Historian:
         """Update a data record builder with standard information."""
         builder.extras.update(self.get_user_info())
         return builder
+
+    def _new_snapshot_depositor(self):
+        return depositors.SnapshotLoader(self)
