@@ -1,6 +1,7 @@
 from . import common_helpers
 from . import builtins
 from . import refs
+from . import testing
 
 
 def get_types():
@@ -9,5 +10,6 @@ def get_types():
     types.extend(builtins.HISTORIAN_TYPES)
     types.extend(common_helpers.HISTORIAN_TYPES)
     types.extend(refs.HISTORIAN_TYPES)
+    types.extend(testing.HISTORIAN_TYPES)
 
     return types
