@@ -1,8 +1,11 @@
+"""This module defines the data record and other objects and functions related to storing things
+in an archive."""
+
 import copy
 from collections import namedtuple
 import datetime
 import typing
-from typing import Optional, Iterable, Sequence, Union, Tuple, Any
+from typing import Optional, Iterable, Sequence, Union, Tuple, Any, Mapping
 import uuid
 
 import pytray.tree
@@ -11,7 +14,7 @@ from . import utils
 
 __all__ = 'OBJ_ID', 'TYPE_ID', 'CREATION_TIME', 'VERSION', 'STATE', 'SNAPSHOT_TIME', \
           'SNAPSHOT_HASH', 'EXTRAS', 'ExtraKeys', 'DELETED', 'DataRecord', 'SnapshotRef', \
-          'DataRecordBuilder'
+          'DataRecordBuilder', 'StateSchema', 'SnapshotId'
 
 OBJ_ID = 'obj_id'
 TYPE_ID = 'type_id'
@@ -23,6 +26,8 @@ SNAPSHOT_HASH = 'snapshot_hash'
 SNAPSHOT_TIME = 'snapshot_time'
 EXTRAS = 'extras'
 
+SchemaEntry = namedtuple('SchemaEntry', 'type_id version')
+
 
 class ExtraKeys:
     # pylint: disable=too-few-public-methods
@@ -45,15 +50,16 @@ EntryPath = Sequence[Union[str, int]]
 EntryInfo = Tuple[EntryPath, TypeId]
 
 
-class SnapshotRef(typing.Generic[IdT]):
-    """Reference information for looking up a snapshot"""
+class SnapshotId(typing.Generic[IdT]):
+    """A snapshot id identifies a particular version of an object (and the corresponding record), it
+    it therefore composed of the object id and the version number."""
 
     #: The type id for references
     TYPE_ID = uuid.UUID('633c7035-64fe-4d87-a91e-3b7abd8a6a28')
 
     def __init__(self, obj_id, version: int):
-        """Create a snapshot reference by passing an object if and version"""
-        super(SnapshotRef, self).__init__()
+        """Create a snapshot id by passing an object id and version"""
+        super().__init__()
         self._obj_id = obj_id
         self._version = version
 
@@ -61,13 +67,13 @@ class SnapshotRef(typing.Generic[IdT]):
         return "{}#{}".format(self._obj_id, self._version)
 
     def __repr__(self):
-        return "SnapshotRef({}, {})".format(self.obj_id, self.version)
+        return "SnapshotId({}, {})".format(self.obj_id, self.version)
 
     def __hash__(self):
         return (self.obj_id, self.version).__hash__()
 
     def __eq__(self, other):
-        if not isinstance(other, SnapshotRef):
+        if not isinstance(other, SnapshotId):
             return False
 
         return self.obj_id == other.obj_id and self.version == other.version
@@ -86,7 +92,7 @@ class SnapshotRef(typing.Generic[IdT]):
         return {'obj_id': self.obj_id, 'version': self.version}
 
 
-Ref = SnapshotRef  # WARNING: This will be removed in 0.11, use SnapshotRef instead!
+SnapshotRef = SnapshotId
 
 
 class DataRecord(
@@ -118,7 +124,7 @@ class DataRecord(
         }
 
     @classmethod
-    def new_builder(cls, **kwargs) -> utils.NamedTupleBuilder:
+    def new_builder(cls, **kwargs) -> 'DataRecordBuilder':
         """Get a builder for a new data record, the version will be set to 0"""
         values = cls.defaults()
         values.update({
@@ -127,7 +133,7 @@ class DataRecord(
             SNAPSHOT_TIME: utils.DefaultFromCall(datetime.datetime.now),
         })
         values.update(kwargs)
-        return utils.NamedTupleBuilder(cls, values)
+        return DataRecordBuilder(cls, values)
 
     @property
     def created_by(self) -> IdT:
@@ -138,48 +144,25 @@ class DataRecord(
         """Does this record represent the object having been deleted"""
         return self.state == DELETED
 
-    def get_reference(self) -> SnapshotRef:
-        """Get a reference for this data record"""
-        return SnapshotRef(self.obj_id, self.version)
+    @property
+    def snapshot_id(self) -> SnapshotId:
+        """The snapshot id for this record"""
+        return SnapshotId(self.obj_id, self.version)
 
-    def get_copied_from(self) -> Optional[SnapshotRef]:
+    def get_copied_from(self) -> Optional[SnapshotId]:
         """Get the reference of the data record this object was originally copied from"""
         obj_ref = self.get_extra(ExtraKeys.COPIED_FROM)
         if obj_ref is None:
             return None
 
-        return SnapshotRef(**obj_ref)
+        return SnapshotId(**obj_ref)
 
     def get_extra(self, name):
         """Convenience function to get an extra from the record, returns None if the extra doesn't
         exist"""
         return self.extras.get(name, None)
 
-    def copy_builder(self, **kwargs) -> utils.NamedTupleBuilder:
-        """Get a copy builder from this DataRecord instance. The following attributes will be
-        copied over:
-
-        * type_id
-        * state [deepcopy]
-        * snapshot_hash
-
-        the version will be set to 0 and the creation time to now
-        """
-        defaults = self.defaults()
-        defaults.update({
-            TYPE_ID: self.type_id,
-            CREATION_TIME: utils.DefaultFromCall(datetime.datetime.now),
-            STATE: copy.deepcopy(self.state),
-            STATE_TYPES: copy.deepcopy(self.state_types),
-            SNAPSHOT_HASH: self.snapshot_hash,
-            VERSION: 0,
-            SNAPSHOT_TIME: utils.DefaultFromCall(datetime.datetime.now),
-        })
-        defaults[EXTRAS][ExtraKeys.COPIED_FROM] = self.get_reference().to_dict()
-        defaults.update(kwargs)
-        return utils.NamedTupleBuilder(type(self), defaults)
-
-    def child_builder(self, **kwargs) -> utils.NamedTupleBuilder:
+    def child_builder(self, **kwargs) -> 'DataRecordBuilder':
         """
         Get a child builder from this DataRecord instance.  The following attributes will be copied
         over:
@@ -198,22 +181,45 @@ class DataRecord(
             CREATION_TIME: self.creation_time,
             VERSION: self.version + 1,
             SNAPSHOT_TIME: utils.DefaultFromCall(datetime.datetime.now),
-            EXTRAS: self.extras.copy(),
+            EXTRAS: copy.deepcopy(self.extras),
         })
         defaults.update(kwargs)
-        return utils.NamedTupleBuilder(type(self), defaults)
+        return DataRecordBuilder(type(self), defaults)
 
-    def get_references(self) -> Iterable[Tuple[EntryPath, SnapshotRef]]:
+    def get_references(self) -> Iterable[Tuple[EntryPath, SnapshotId]]:
         """Get all the references to other objects contained in this record"""
         references = []
         if self.state_types is not None and self.state is not None:
-            for entry_info in filter(lambda entry: entry[1] == SnapshotRef.TYPE_ID,
+            for entry_info in filter(lambda entry: entry[1] == SnapshotId.TYPE_ID,
                                      self.state_types):
                 path = entry_info[0]
-                references.append((path, SnapshotRef(**pytray.tree.get_by_path(self.state, path))))
+                sid_info = pytray.tree.get_by_path(self.state, path)
+                if sid_info is not None:
+                    sid = SnapshotId(**sid_info)
+                    references.append((path, sid))
         return references
 
+    def get_state_schema(self) -> 'StateSchema':
+        """Get the schema for the state.  This contains the types and versions for each member of
+        the state"""
+        schema = {}
+        for entry in self.state_types:
+            path = tuple(entry[0])
+            type_id = entry[1]
+            version = None
+            if len(entry) > 2:
+                version = entry[2]
+            schema[path] = SchemaEntry(type_id, version)
 
+        return schema
+
+
+# State schema is a mapping where they key is a tuple containing a path to the entry in the schema
+# and the value is information about that entry, including its type
+# The reason a tuple used (instead of, say, a string) is that this way we can preserve the python
+# type of they key e.g. (str, int, int str) could be used to reference a dictionary, then index in
+# list, then index in list and then a string in a dictionary.
+StateSchema = Mapping[tuple, SchemaEntry]
 DataRecordBuilder = utils.NamedTupleBuilder[DataRecord]  # pylint: disable=invalid-name
 
 
