@@ -1,3 +1,4 @@
+import gc
 import uuid
 
 import pytest
@@ -6,6 +7,8 @@ import mincepy
 import mincepy.builtins
 from mincepy.testing import Car, Garage, Cycle
 
+# pylint: disable=invalid-name, too-few-public-methods
+
 
 def test_basic_save_load(historian: mincepy.Historian):
     car = Car('nissan', 'white')
@@ -148,21 +151,6 @@ def test_storing_internal_object(historian: mincepy.Historian):
     assert loaded_mike.car == ferrari  # But values should match
 
 
-def test_copy(historian: mincepy.Historian):
-    car = Car('zonda')
-
-    historian.save(car)
-    car_copy = historian.copy(car)
-    assert car == car_copy
-    assert car is not car_copy
-
-    record = historian.get_current_record(car)
-    copy_record = historian.get_current_record(car_copy)
-
-    assert record is not copy_record
-    assert copy_record.get_copied_from() == record.get_reference()
-
-
 def test_load_unknown_object(mongodb_archive, historian: mincepy.Historian):
     """Make up an ID an try to load it"""
     obj_id = mongodb_archive.create_archive_id()
@@ -175,6 +163,7 @@ def test_cyclic_ref_simple(historian: mincepy.Historian):
     a.ref = a  # Cycle complete
     a_id = historian.save(a)
     del a
+    gc.collect()  # Force a garbage collection
 
     loaded_a = historian.load(a_id)
     assert loaded_a.ref is loaded_a
@@ -187,10 +176,16 @@ def test_cyclic_ref_complex(historian: mincepy.Historian):
 
     a_id = historian.save(a)
     del a
+
     loaded_a = historian.load(a_id)
     assert loaded_a.ref is b
     assert b.ref is loaded_a
 
+    del loaded_a, b
+    gc.collect()  # Force a garbage collection
+    reloaded_a = historian.load(a_id)
+    assert isinstance(reloaded_a.ref, Cycle)
+
 
 def test_transaction_rollback(historian: mincepy.Historian):
     ferrari = Car('ferrari')
@@ -239,7 +234,7 @@ def test_replace_simple(historian: mincepy.Historian):
 def test_replace_invalid(historian: mincepy.Historian):
     honda = Car('honda', 'yellow')
     historian.save(honda)
-    with pytest.raises(AssertionError):
+    with pytest.raises(TypeError):
         historian.replace(honda, Garage())
 
 
