@@ -6,12 +6,12 @@ from mincepy.testing import Car
 def test_transaction_snapshots(historian: mincepy.Historian):
     ferrari = Car('ferrari')
     historian.save(ferrari)
-    ferrari_ref = historian.get_snapshot_ref(ferrari)
+    ferrari_sid = historian.get_snapshot_id(ferrari)
 
     with historian.transaction():
-        ferrari_snapshot_1 = historian.load_snapshot(ferrari_ref)
+        ferrari_snapshot_1 = historian.load_snapshot(ferrari_sid)
         with historian.transaction():
-            ferrari_snapshot_2 = historian.load_snapshot(ferrari_ref)
+            ferrari_snapshot_2 = historian.load_snapshot(ferrari_sid)
             # Reference wise they should be unequal
             assert ferrari_snapshot_1 is not ferrari_snapshot_2
             assert ferrari is not ferrari_snapshot_1
@@ -22,7 +22,7 @@ def test_transaction_snapshots(historian: mincepy.Historian):
             assert ferrari == ferrari_snapshot_2
 
         # Now check within the same transaction the result is the same
-        ferrari_snapshot_2 = historian.load_snapshot(ferrari_ref)
+        ferrari_snapshot_2 = historian.load_snapshot(ferrari_sid)
         # Reference wise they should be unequal
         assert ferrari_snapshot_1 is not ferrari_snapshot_2
         assert ferrari is not ferrari_snapshot_1
@@ -104,3 +104,31 @@ def test_to_obj_id(historian: mincepy.Historian):
     assert historian.to_obj_id(car_id) == car_id
     assert historian.to_obj_id(car) == car_id
     assert historian.to_obj_id(str(car_id)) == car_id
+
+
+def test_copy(historian: mincepy.Historian):
+    car = Car('zonda')
+
+    historian.save(car)
+    car_copy = mincepy.copy(car)
+    assert car == car_copy
+    assert car is not car_copy
+    car_copy.save()
+
+    record = historian.get_current_record(car)
+    copy_record = historian.get_current_record(car_copy)
+
+    assert record is not copy_record
+    assert copy_record.get_copied_from() == record.snapshot_id
+
+
+def test_copy_unsaved(historian: mincepy.Historian):
+    car = Car('porsche', 'silver')
+    car_copy = mincepy.copy(car)
+
+    assert car_copy is not car
+    assert car == car_copy
+
+    # The cars should not be saved
+    assert not historian.is_known(car)
+    assert not historian.is_known(car_copy)
